import System;
import System.Collections;
import System.Drawing;
import System.Drawing.Imaging;
import System.IO;
import System.Reflection;

class TypeComparer implements IComparer {
    public function Compare(x:Object, y:Object) : int {
        return Type(x).FullName.CompareTo(Type(y).FullName);
    }
}


class AssemblyGraph {
    public static function RenderGraph(asm:String, save:String) : Boolean {
        var as:ArrayList = new ArrayList();
        var ns:Hashtable = new Hashtable();
        var nst:Hashtable = new Hashtable();
        var curns:String;
        var assembly:Assembly = null;
        var arrayList:ArrayList;
        var types:Type[];
        var props:PropertyInfo[];
        var methd:MethodInfo[];
        var contr:ConstructorInfo[];
        var felds:FieldInfo[];
        var totalElements:int = 0;
        var i:int;
        var j:int;
        var k:int;
        
        var normalFont:Font = new Font("Arial", 8);
        var headerFont:Font = new Font(normalFont, FontStyle.Bold);
        
        var headerBrush:Brush = Brushes.LightGray;
        
        try {
            assembly = Assembly.Load(asm);
            types = assembly.GetTypes();
            
            for(i = 0; i < types.Length; i++) {
                if ( types[i].Namespace == null ) {
                    curns = "(Global)";
                } else {
                    curns = types[i].Namespace;
                }
                
                arrayList = ns[curns];
                if ( arrayList == null ) {
                    nst[curns] = 1;
                    as.Add(curns); // For sorting
                    arrayList = new ArrayList();
                }
                
                nst[curns]++;
                arrayList.Add(types[i]);

                props = types[i].GetProperties(BindingFlags.Instance | BindingFlags.Static | BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.DeclaredOnly);
                methd = types[i].GetMethods(BindingFlags.Instance | BindingFlags.Static | BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.DeclaredOnly);
                contr = types[i].GetConstructors(BindingFlags.Instance | BindingFlags.Static | BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.DeclaredOnly);
                felds = types[i].GetFields(BindingFlags.Instance | BindingFlags.Static | BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.DeclaredOnly);

                if ( props.Length > 0 ) { nst[curns] += props.Length + 1; }
                if ( methd.Length > 0 ) { nst[curns] += methd.Length + 1; }
                if ( contr.Length > 0 ) { nst[curns] += contr.Length + 1; }
                if ( felds.Length > 0 ) { nst[curns] += felds.Length + 1; }

                ns[curns] = arrayList;
            }
            
            var fore:Brush;
            var back:Brush;
            
            as.Sort();
            var sw:StreamWriter = new StreamWriter(save + ".html", false);
            sw.WriteLine("<html>\n <head>\n  <title>" + save + "</title>\n </head>\n <body>\n");
            sw.WriteLine("  <h1>Namespace</h1>");
            for(i = 0; i < as.Count; i++) {
                sw.WriteLine("  <a href=\"" + save + "-" + as[i] + ".jpg" + "\">" + as[i] + "</a><br>");
            }
            sw.WriteLine(" </body>\n</html>");
            sw.Close();
            
            for(i = 0; i < as.Count; i++) {
                var bmp:Bitmap = new Bitmap(400, int(nst[as[i]]*20));
                var gfx:Graphics = Graphics.FromImage(Image(bmp));
                var curElement:int = 0;

                DrawTag(as[i], Brushes.White, Brushes.Navy, null, gfx, headerFont, curElement, 0);
                arrayList = ns[as[i]];
                arrayList.Sort(new TypeComparer());
                
                curElement++;
                for(j = 0; j < arrayList.Count; j++) {
                    if ( Type(arrayList[j]).IsPublic ) {
                        back = Brushes.Lime;
                    } else {
                        back = Brushes.Red;
                    }
                    DrawTag(Type(arrayList[j]).Name, Brushes.Black, Brushes.DarkGray, back, gfx, headerFont, curElement, 1);
                    curElement++;
                    
                    props = Type(arrayList[j]).GetProperties(BindingFlags.Instance | BindingFlags.Static | BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.DeclaredOnly);
                    if ( props.Length > 0 ) {
                        DrawTag("Properties", Brushes.Black, headerBrush, null, gfx, headerFont, curElement, 2);
                        curElement++;
                    }

                    for(k = 0; k < props.Length; k++) {
                        var getter:MethodInfo = props[k].GetGetMethod(true);
                        var setter:MethodInfo = props[k].GetSetMethod(true);
                        var vis:MethodInfo = null;
                        
                        if ( getter != null ) {
                            vis = getter;
                        } else {
                            vis = setter;
                        }

                        if ( vis.IsPublic ) {
                            back = Brushes.Lime;
                        } else if ( vis.IsPrivate ) {
                            back = Brushes.Red;
                        } else if ( vis.IsFamily || vis.IsAssembly ) {
                            back = Brushes.Orange;
                        } else {
                            back = Brushes.Cyan;
                        }
                            
                        DrawTag(props[k].Name + " " + ((getter == null) ? "Set" : (setter != null) ? "Get;Set" : "Get"), Brushes.Black, Brushes.White, back, gfx, normalFont, curElement, 3);
                        curElement++;
                    }

                    felds = Type(arrayList[j]).GetFields(BindingFlags.Instance | BindingFlags.Static | BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.DeclaredOnly);
                    if ( felds.Length > 0 ) {
                        DrawTag("Fields", Brushes.Black, headerBrush, null, gfx, headerFont, curElement, 2);
                        curElement++;
                    }

                    for(k = 0; k < felds.Length; k++) {
                        if ( felds[k].IsPublic ) {
                            back = Brushes.Lime;
                        } else if ( felds[k].IsPrivate ) {
                            back = Brushes.Red;
                        } else if ( felds[k].IsFamily || felds[k].IsAssembly ) {
                            back = Brushes.Orange;
                        } else {
                            back = Brushes.Cyan;
                        }
                        
                        DrawTag(felds[k].Name, Brushes.Black, Brushes.White, back, gfx, normalFont, curElement, 3);
                        curElement++;
                    }

                    methd = Type(arrayList[j]).GetMethods(BindingFlags.Instance | BindingFlags.Static | BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.DeclaredOnly);
                    if ( methd.Length > 0 ) {
                        DrawTag("Methods", Brushes.Black, headerBrush, null, gfx, headerFont, curElement, 2);
                        curElement++;
                    }

                    for(k = 0; k < methd.Length; k++) {
                        if ( methd[k].IsPublic ) {
                            back = Brushes.Lime;
                        } else if ( methd[k].IsPrivate ) {
                            back = Brushes.Red;
                        } else if ( methd[k].IsFamily || methd[k].IsAssembly ) {
                            back = Brushes.Orange;
                        } else {
                            back = Brushes.Cyan;
                        }
                        
                        DrawTag(methd[k].Name, Brushes.Black, Brushes.White, back, gfx, normalFont, curElement, 3);
                        curElement++;
                    }
                    

                    contr = Type(arrayList[j]).GetConstructors(BindingFlags.Instance | BindingFlags.Static | BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.DeclaredOnly);
                    if ( contr.Length > 0 ) {
                        DrawTag("Constructors", Brushes.Black, headerBrush, null, gfx, headerFont, curElement, 2);
                        curElement++;
                    }
                    
                    for(k = 0; k < contr.Length; k++) {
                        if ( contr[k].IsPublic ) {
                            back = Brushes.Lime;
                        } else if ( contr[k].IsPrivate ) {
                            back = Brushes.Red;
                        } else if ( contr[k].IsFamily || contr[k].IsAssembly ) {
                            back = Brushes.Orange;
                        } else {
                            back = Brushes.Cyan;
                        }
                        
                        DrawTag(contr[k].Name, Brushes.Black, Brushes.White, back, gfx, normalFont, curElement, 3);
                        curElement++;
                    }
                }

                gfx.Dispose();
                bmp.Save(save + "-" + as[i] + ".jpg", ImageFormat.Jpeg);
            }
            
        } catch(e:Exception) {
            Console.WriteLine(e.ToString());
        }
    }
    
    private static function DrawTag(tag:String, fore:Brush, back:Brush, mark:Brush, gfx:Graphics, font:Font, curElement:int, indent:int) : void {
        indent = (indent * 20) + 5;
        curElement = curElement * 20;
        gfx.FillRectangle(back, indent, curElement, int(400 - (indent+5)), 18);
        if ( mark != null ) {
            gfx.FillRectangle(mark, 375, curElement, 20, 18);
        }
        gfx.DrawString(tag, font, fore, (indent+5), (curElement+2));
    }
}

var Args:String[] = Environment.GetCommandLineArgs();
if ( Args.Length > 2 ) {
    AssemblyGraph.RenderGraph(Args[1] /* Asssembly */, Args[2] /* Save File */);
}

